/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by main. DO NOT EDIT.

package v1

import (
	"context"
	"time"

	"github.com/rancher/lasso/pkg/client"
	"github.com/rancher/lasso/pkg/controller"
	"github.com/rancher/wrangler/pkg/generic"
	v1 "k8s.io/api/admissionregistration/v1"
	"k8s.io/apimachinery/pkg/api/equality"
	"k8s.io/apimachinery/pkg/api/errors"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/labels"
	"k8s.io/apimachinery/pkg/runtime"
	"k8s.io/apimachinery/pkg/runtime/schema"
	"k8s.io/apimachinery/pkg/types"
	utilruntime "k8s.io/apimachinery/pkg/util/runtime"
	"k8s.io/apimachinery/pkg/watch"
	"k8s.io/client-go/tools/cache"
)

type MutatingWebhookConfigurationHandler func(string, *v1.MutatingWebhookConfiguration) (*v1.MutatingWebhookConfiguration, error)

type MutatingWebhookConfigurationController interface {
	generic.ControllerMeta
	MutatingWebhookConfigurationClient

	OnChange(ctx context.Context, name string, sync MutatingWebhookConfigurationHandler)
	OnRemove(ctx context.Context, name string, sync MutatingWebhookConfigurationHandler)
	Enqueue(name string)
	EnqueueAfter(name string, duration time.Duration)

	Cache() MutatingWebhookConfigurationCache
}

type MutatingWebhookConfigurationClient interface {
	Create(*v1.MutatingWebhookConfiguration) (*v1.MutatingWebhookConfiguration, error)
	Update(*v1.MutatingWebhookConfiguration) (*v1.MutatingWebhookConfiguration, error)

	Delete(name string, options *metav1.DeleteOptions) error
	Get(name string, options metav1.GetOptions) (*v1.MutatingWebhookConfiguration, error)
	List(opts metav1.ListOptions) (*v1.MutatingWebhookConfigurationList, error)
	Watch(opts metav1.ListOptions) (watch.Interface, error)
	Patch(name string, pt types.PatchType, data []byte, subresources ...string) (result *v1.MutatingWebhookConfiguration, err error)
}

type MutatingWebhookConfigurationCache interface {
	Get(name string) (*v1.MutatingWebhookConfiguration, error)
	List(selector labels.Selector) ([]*v1.MutatingWebhookConfiguration, error)

	AddIndexer(indexName string, indexer MutatingWebhookConfigurationIndexer)
	GetByIndex(indexName, key string) ([]*v1.MutatingWebhookConfiguration, error)
}

type MutatingWebhookConfigurationIndexer func(obj *v1.MutatingWebhookConfiguration) ([]string, error)

type mutatingWebhookConfigurationController struct {
	controller    controller.SharedController
	client        *client.Client
	gvk           schema.GroupVersionKind
	groupResource schema.GroupResource
}

func NewMutatingWebhookConfigurationController(gvk schema.GroupVersionKind, resource string, namespaced bool, controller controller.SharedControllerFactory) MutatingWebhookConfigurationController {
	c := controller.ForResourceKind(gvk.GroupVersion().WithResource(resource), gvk.Kind, namespaced)
	return &mutatingWebhookConfigurationController{
		controller: c,
		client:     c.Client(),
		gvk:        gvk,
		groupResource: schema.GroupResource{
			Group:    gvk.Group,
			Resource: resource,
		},
	}
}

func FromMutatingWebhookConfigurationHandlerToHandler(sync MutatingWebhookConfigurationHandler) generic.Handler {
	return func(key string, obj runtime.Object) (ret runtime.Object, err error) {
		var v *v1.MutatingWebhookConfiguration
		if obj == nil {
			v, err = sync(key, nil)
		} else {
			v, err = sync(key, obj.(*v1.MutatingWebhookConfiguration))
		}
		if v == nil {
			return nil, err
		}
		return v, err
	}
}

func (c *mutatingWebhookConfigurationController) Updater() generic.Updater {
	return func(obj runtime.Object) (runtime.Object, error) {
		newObj, err := c.Update(obj.(*v1.MutatingWebhookConfiguration))
		if newObj == nil {
			return nil, err
		}
		return newObj, err
	}
}

func UpdateMutatingWebhookConfigurationDeepCopyOnChange(client MutatingWebhookConfigurationClient, obj *v1.MutatingWebhookConfiguration, handler func(obj *v1.MutatingWebhookConfiguration) (*v1.MutatingWebhookConfiguration, error)) (*v1.MutatingWebhookConfiguration, error) {
	if obj == nil {
		return obj, nil
	}

	copyObj := obj.DeepCopy()
	newObj, err := handler(copyObj)
	if newObj != nil {
		copyObj = newObj
	}
	if obj.ResourceVersion == copyObj.ResourceVersion && !equality.Semantic.DeepEqual(obj, copyObj) {
		return client.Update(copyObj)
	}

	return copyObj, err
}

func (c *mutatingWebhookConfigurationController) AddGenericHandler(ctx context.Context, name string, handler generic.Handler) {
	c.controller.RegisterHandler(ctx, name, controller.SharedControllerHandlerFunc(handler))
}

func (c *mutatingWebhookConfigurationController) AddGenericRemoveHandler(ctx context.Context, name string, handler generic.Handler) {
	c.AddGenericHandler(ctx, name, generic.NewRemoveHandler(name, c.Updater(), handler))
}

func (c *mutatingWebhookConfigurationController) OnChange(ctx context.Context, name string, sync MutatingWebhookConfigurationHandler) {
	c.AddGenericHandler(ctx, name, FromMutatingWebhookConfigurationHandlerToHandler(sync))
}

func (c *mutatingWebhookConfigurationController) OnRemove(ctx context.Context, name string, sync MutatingWebhookConfigurationHandler) {
	c.AddGenericHandler(ctx, name, generic.NewRemoveHandler(name, c.Updater(), FromMutatingWebhookConfigurationHandlerToHandler(sync)))
}

func (c *mutatingWebhookConfigurationController) Enqueue(name string) {
	c.controller.Enqueue("", name)
}

func (c *mutatingWebhookConfigurationController) EnqueueAfter(name string, duration time.Duration) {
	c.controller.EnqueueAfter("", name, duration)
}

func (c *mutatingWebhookConfigurationController) Informer() cache.SharedIndexInformer {
	return c.controller.Informer()
}

func (c *mutatingWebhookConfigurationController) GroupVersionKind() schema.GroupVersionKind {
	return c.gvk
}

func (c *mutatingWebhookConfigurationController) Cache() MutatingWebhookConfigurationCache {
	return &mutatingWebhookConfigurationCache{
		indexer:  c.Informer().GetIndexer(),
		resource: c.groupResource,
	}
}

func (c *mutatingWebhookConfigurationController) Create(obj *v1.MutatingWebhookConfiguration) (*v1.MutatingWebhookConfiguration, error) {
	result := &v1.MutatingWebhookConfiguration{}
	return result, c.client.Create(context.TODO(), "", obj, result, metav1.CreateOptions{})
}

func (c *mutatingWebhookConfigurationController) Update(obj *v1.MutatingWebhookConfiguration) (*v1.MutatingWebhookConfiguration, error) {
	result := &v1.MutatingWebhookConfiguration{}
	return result, c.client.Update(context.TODO(), "", obj, result, metav1.UpdateOptions{})
}

func (c *mutatingWebhookConfigurationController) Delete(name string, options *metav1.DeleteOptions) error {
	if options == nil {
		options = &metav1.DeleteOptions{}
	}
	return c.client.Delete(context.TODO(), "", name, *options)
}

func (c *mutatingWebhookConfigurationController) Get(name string, options metav1.GetOptions) (*v1.MutatingWebhookConfiguration, error) {
	result := &v1.MutatingWebhookConfiguration{}
	return result, c.client.Get(context.TODO(), "", name, result, options)
}

func (c *mutatingWebhookConfigurationController) List(opts metav1.ListOptions) (*v1.MutatingWebhookConfigurationList, error) {
	result := &v1.MutatingWebhookConfigurationList{}
	return result, c.client.List(context.TODO(), "", result, opts)
}

func (c *mutatingWebhookConfigurationController) Watch(opts metav1.ListOptions) (watch.Interface, error) {
	return c.client.Watch(context.TODO(), "", opts)
}

func (c *mutatingWebhookConfigurationController) Patch(name string, pt types.PatchType, data []byte, subresources ...string) (*v1.MutatingWebhookConfiguration, error) {
	result := &v1.MutatingWebhookConfiguration{}
	return result, c.client.Patch(context.TODO(), "", name, pt, data, result, metav1.PatchOptions{}, subresources...)
}

type mutatingWebhookConfigurationCache struct {
	indexer  cache.Indexer
	resource schema.GroupResource
}

func (c *mutatingWebhookConfigurationCache) Get(name string) (*v1.MutatingWebhookConfiguration, error) {
	obj, exists, err := c.indexer.GetByKey(name)
	if err != nil {
		return nil, err
	}
	if !exists {
		return nil, errors.NewNotFound(c.resource, name)
	}
	return obj.(*v1.MutatingWebhookConfiguration), nil
}

func (c *mutatingWebhookConfigurationCache) List(selector labels.Selector) (ret []*v1.MutatingWebhookConfiguration, err error) {

	err = cache.ListAll(c.indexer, selector, func(m interface{}) {
		ret = append(ret, m.(*v1.MutatingWebhookConfiguration))
	})

	return ret, err
}

func (c *mutatingWebhookConfigurationCache) AddIndexer(indexName string, indexer MutatingWebhookConfigurationIndexer) {
	utilruntime.Must(c.indexer.AddIndexers(map[string]cache.IndexFunc{
		indexName: func(obj interface{}) (strings []string, e error) {
			return indexer(obj.(*v1.MutatingWebhookConfiguration))
		},
	}))
}

func (c *mutatingWebhookConfigurationCache) GetByIndex(indexName, key string) (result []*v1.MutatingWebhookConfiguration, err error) {
	objs, err := c.indexer.ByIndex(indexName, key)
	if err != nil {
		return nil, err
	}
	result = make([]*v1.MutatingWebhookConfiguration, 0, len(objs))
	for _, obj := range objs {
		result = append(result, obj.(*v1.MutatingWebhookConfiguration))
	}
	return result, nil
}
