/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by main. DO NOT EDIT.

package v1

import (
	"context"
	"time"

	"github.com/rancher/lasso/pkg/client"
	"github.com/rancher/lasso/pkg/controller"
	"github.com/rancher/wrangler/pkg/generic"
	v1 "k8s.io/api/networking/v1"
	"k8s.io/apimachinery/pkg/api/equality"
	"k8s.io/apimachinery/pkg/api/errors"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/labels"
	"k8s.io/apimachinery/pkg/runtime"
	"k8s.io/apimachinery/pkg/runtime/schema"
	"k8s.io/apimachinery/pkg/types"
	utilruntime "k8s.io/apimachinery/pkg/util/runtime"
	"k8s.io/apimachinery/pkg/watch"
	"k8s.io/client-go/tools/cache"
)

type NetworkPolicyHandler func(string, *v1.NetworkPolicy) (*v1.NetworkPolicy, error)

type NetworkPolicyController interface {
	generic.ControllerMeta
	NetworkPolicyClient

	OnChange(ctx context.Context, name string, sync NetworkPolicyHandler)
	OnRemove(ctx context.Context, name string, sync NetworkPolicyHandler)
	Enqueue(namespace, name string)
	EnqueueAfter(namespace, name string, duration time.Duration)

	Cache() NetworkPolicyCache
}

type NetworkPolicyClient interface {
	Create(*v1.NetworkPolicy) (*v1.NetworkPolicy, error)
	Update(*v1.NetworkPolicy) (*v1.NetworkPolicy, error)

	Delete(namespace, name string, options *metav1.DeleteOptions) error
	Get(namespace, name string, options metav1.GetOptions) (*v1.NetworkPolicy, error)
	List(namespace string, opts metav1.ListOptions) (*v1.NetworkPolicyList, error)
	Watch(namespace string, opts metav1.ListOptions) (watch.Interface, error)
	Patch(namespace, name string, pt types.PatchType, data []byte, subresources ...string) (result *v1.NetworkPolicy, err error)
}

type NetworkPolicyCache interface {
	Get(namespace, name string) (*v1.NetworkPolicy, error)
	List(namespace string, selector labels.Selector) ([]*v1.NetworkPolicy, error)

	AddIndexer(indexName string, indexer NetworkPolicyIndexer)
	GetByIndex(indexName, key string) ([]*v1.NetworkPolicy, error)
}

type NetworkPolicyIndexer func(obj *v1.NetworkPolicy) ([]string, error)

type networkPolicyController struct {
	controller    controller.SharedController
	client        *client.Client
	gvk           schema.GroupVersionKind
	groupResource schema.GroupResource
}

func NewNetworkPolicyController(gvk schema.GroupVersionKind, resource string, namespaced bool, controller controller.SharedControllerFactory) NetworkPolicyController {
	c := controller.ForResourceKind(gvk.GroupVersion().WithResource(resource), gvk.Kind, namespaced)
	return &networkPolicyController{
		controller: c,
		client:     c.Client(),
		gvk:        gvk,
		groupResource: schema.GroupResource{
			Group:    gvk.Group,
			Resource: resource,
		},
	}
}

func FromNetworkPolicyHandlerToHandler(sync NetworkPolicyHandler) generic.Handler {
	return func(key string, obj runtime.Object) (ret runtime.Object, err error) {
		var v *v1.NetworkPolicy
		if obj == nil {
			v, err = sync(key, nil)
		} else {
			v, err = sync(key, obj.(*v1.NetworkPolicy))
		}
		if v == nil {
			return nil, err
		}
		return v, err
	}
}

func (c *networkPolicyController) Updater() generic.Updater {
	return func(obj runtime.Object) (runtime.Object, error) {
		newObj, err := c.Update(obj.(*v1.NetworkPolicy))
		if newObj == nil {
			return nil, err
		}
		return newObj, err
	}
}

func UpdateNetworkPolicyDeepCopyOnChange(client NetworkPolicyClient, obj *v1.NetworkPolicy, handler func(obj *v1.NetworkPolicy) (*v1.NetworkPolicy, error)) (*v1.NetworkPolicy, error) {
	if obj == nil {
		return obj, nil
	}

	copyObj := obj.DeepCopy()
	newObj, err := handler(copyObj)
	if newObj != nil {
		copyObj = newObj
	}
	if obj.ResourceVersion == copyObj.ResourceVersion && !equality.Semantic.DeepEqual(obj, copyObj) {
		return client.Update(copyObj)
	}

	return copyObj, err
}

func (c *networkPolicyController) AddGenericHandler(ctx context.Context, name string, handler generic.Handler) {
	c.controller.RegisterHandler(ctx, name, controller.SharedControllerHandlerFunc(handler))
}

func (c *networkPolicyController) AddGenericRemoveHandler(ctx context.Context, name string, handler generic.Handler) {
	c.AddGenericHandler(ctx, name, generic.NewRemoveHandler(name, c.Updater(), handler))
}

func (c *networkPolicyController) OnChange(ctx context.Context, name string, sync NetworkPolicyHandler) {
	c.AddGenericHandler(ctx, name, FromNetworkPolicyHandlerToHandler(sync))
}

func (c *networkPolicyController) OnRemove(ctx context.Context, name string, sync NetworkPolicyHandler) {
	c.AddGenericHandler(ctx, name, generic.NewRemoveHandler(name, c.Updater(), FromNetworkPolicyHandlerToHandler(sync)))
}

func (c *networkPolicyController) Enqueue(namespace, name string) {
	c.controller.Enqueue(namespace, name)
}

func (c *networkPolicyController) EnqueueAfter(namespace, name string, duration time.Duration) {
	c.controller.EnqueueAfter(namespace, name, duration)
}

func (c *networkPolicyController) Informer() cache.SharedIndexInformer {
	return c.controller.Informer()
}

func (c *networkPolicyController) GroupVersionKind() schema.GroupVersionKind {
	return c.gvk
}

func (c *networkPolicyController) Cache() NetworkPolicyCache {
	return &networkPolicyCache{
		indexer:  c.Informer().GetIndexer(),
		resource: c.groupResource,
	}
}

func (c *networkPolicyController) Create(obj *v1.NetworkPolicy) (*v1.NetworkPolicy, error) {
	result := &v1.NetworkPolicy{}
	return result, c.client.Create(context.TODO(), obj.Namespace, obj, result, metav1.CreateOptions{})
}

func (c *networkPolicyController) Update(obj *v1.NetworkPolicy) (*v1.NetworkPolicy, error) {
	result := &v1.NetworkPolicy{}
	return result, c.client.Update(context.TODO(), obj.Namespace, obj, result, metav1.UpdateOptions{})
}

func (c *networkPolicyController) Delete(namespace, name string, options *metav1.DeleteOptions) error {
	if options == nil {
		options = &metav1.DeleteOptions{}
	}
	return c.client.Delete(context.TODO(), namespace, name, *options)
}

func (c *networkPolicyController) Get(namespace, name string, options metav1.GetOptions) (*v1.NetworkPolicy, error) {
	result := &v1.NetworkPolicy{}
	return result, c.client.Get(context.TODO(), namespace, name, result, options)
}

func (c *networkPolicyController) List(namespace string, opts metav1.ListOptions) (*v1.NetworkPolicyList, error) {
	result := &v1.NetworkPolicyList{}
	return result, c.client.List(context.TODO(), namespace, result, opts)
}

func (c *networkPolicyController) Watch(namespace string, opts metav1.ListOptions) (watch.Interface, error) {
	return c.client.Watch(context.TODO(), namespace, opts)
}

func (c *networkPolicyController) Patch(namespace, name string, pt types.PatchType, data []byte, subresources ...string) (*v1.NetworkPolicy, error) {
	result := &v1.NetworkPolicy{}
	return result, c.client.Patch(context.TODO(), namespace, name, pt, data, result, metav1.PatchOptions{}, subresources...)
}

type networkPolicyCache struct {
	indexer  cache.Indexer
	resource schema.GroupResource
}

func (c *networkPolicyCache) Get(namespace, name string) (*v1.NetworkPolicy, error) {
	obj, exists, err := c.indexer.GetByKey(namespace + "/" + name)
	if err != nil {
		return nil, err
	}
	if !exists {
		return nil, errors.NewNotFound(c.resource, name)
	}
	return obj.(*v1.NetworkPolicy), nil
}

func (c *networkPolicyCache) List(namespace string, selector labels.Selector) (ret []*v1.NetworkPolicy, err error) {

	err = cache.ListAllByNamespace(c.indexer, namespace, selector, func(m interface{}) {
		ret = append(ret, m.(*v1.NetworkPolicy))
	})

	return ret, err
}

func (c *networkPolicyCache) AddIndexer(indexName string, indexer NetworkPolicyIndexer) {
	utilruntime.Must(c.indexer.AddIndexers(map[string]cache.IndexFunc{
		indexName: func(obj interface{}) (strings []string, e error) {
			return indexer(obj.(*v1.NetworkPolicy))
		},
	}))
}

func (c *networkPolicyCache) GetByIndex(indexName, key string) (result []*v1.NetworkPolicy, err error) {
	objs, err := c.indexer.ByIndex(indexName, key)
	if err != nil {
		return nil, err
	}
	result = make([]*v1.NetworkPolicy, 0, len(objs))
	for _, obj := range objs {
		result = append(result, obj.(*v1.NetworkPolicy))
	}
	return result, nil
}
